@@ -79,6 +79,11 @@
 
 #include <glib.h>
 
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+#define SN_API_NOT_YET_FROZEN 1
+#include <libsn/sn.h>
+#endif
+
 #include <libxfce4util/libxfce4util.h>
 
 #include <libxfce4mcs/mcs-channel.h>
@@ -416,6 +421,44 @@ mcs_manager_get_mcs_window (McsManager *manager)
 }
 #endif
 
+static gchar *
+mcs_manager_show_extract_name(Display *display,
+                              int screen,
+                              Window w,
+                              guchar *data)
+{
+    gchar *name = NULL, *p;
+    
+    p = strstr((gchar *)data, "|");
+    if(p == (gchar *)data)
+        name = g_strdup("");
+    else if(p) {
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+        SnDisplay *sndpy;
+        SnLauncheeContext *snctx;
+#endif
+        gchar **name_parts = g_strsplit((gchar *)data, "|", 2);
+        
+        name = name_parts[0];
+        
+#ifdef HAVE_LIBSTARTUP_NOTIFICATION
+        sndpy = sn_display_new(display,
+                               (SnDisplayErrorTrapPush)sn_display_error_trap_push,
+                               (SnDisplayErrorTrapPop)sn_display_error_trap_pop);
+        snctx = sn_launchee_context_new(sndpy, screen, name_parts[1]);
+        if(snctx) {
+            sn_launchee_context_setup_window(snctx, w);
+            sn_launchee_context_complete(snctx);
+            sn_launchee_context_unref(snctx);
+        }
+        sn_display_unref(sndpy);
+#endif
+    } else
+        name = g_strdup((gchar *)data);
+    
+    return name;
+}
+
 gboolean
 mcs_manager_process_event (McsManager *manager, 
                            XEvent     *xev)
@@ -453,7 +496,16 @@ mcs_manager_process_event (McsManager *manager,
             if (result == Success)
             {
                 if (type == manager->show_atom && manager->show)
-                    manager->show (data, manager->cb_data);
+                {
+                    gchar *name;
+                    
+                    name = mcs_manager_show_extract_name(manager->display,
+                                                         screen,
+                                                         manager->mcs_windows[screen],
+                                                         data);
+                    manager->show (name, manager->cb_data);
+                    g_free(name);
+                }
 
                 XFree (data);
                 return True;
